#!/bin/sh

## Define function
# runiraf (iraf command) (output logfile)
runiraf(){
  TMPLOG=$TMPPATH/tmp.log

  cat $1 >> $3
  cat $1 | cl > $TMPLOG
  cat $TMPLOG >> $2
  echo "" >> $2
  ERRSTAT=`grep ERROR $TMPLOG | gawk 'END{print(NR)}'`

  if [ $ERRSTAT -ne 0 ]; then
   echo "# Error in IRAF ... Exiting #"
   echo "# Check $2. #" 
   if [ $WFCAMDEBUG -eq 1 ]; then 
     \rm -r $TMPPATH
   fi
   exit 1
  fi
}

#Check Environmental Parameters
if [ `env | grep WFCAMDATA |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMDATA not defined!!"
 exit 1
fi

if [ `env | grep WFCAMDATE |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMDATE not defined!!"
 exit 1
fi

if [ `env | grep WFCAMLOG |gawk 'END{print(NR)}'` -eq 0 ]; then
 echo "Error : Environmental parameter WFCAMLOG not defined!!"
 exit 1
fi

if [ `env | grep MASKRADIUS |gawk 'END{print(NR)}'` -eq 0 ]; then
 MASKRADIUS=6 
fi

MASKRADIUS2=`echo $MASKRADIUS| gawk '{print(int($1*1.4))}'`

if [ `env | grep MASKTHRESHO |gawk 'END{print(NR)}'` -eq 0 ]; then
 MASKTHRESHO=7 
fi

MASKTHRESHO2=`echo $MASKTHRESHO| gawk '{print(int($1*4))}'`

## Set DISPLAY Parameter
# WFCAMDISP=0; Display all images
#            1; Display minimum images
#            2; Display No images
if [ `env | grep WFCAMDISP |gawk 'END{print(NR)}'` -eq 0 ]; then
 WFCAMDISP=1 
fi

# Debug Parameter 
# WFCAMDEBUG=1; erase all temporary files 
#            0; leave all temporary files 
if [ `env | grep WFCAMDEBUG |gawk 'END{print(NR)}'` -eq 0 ]; then
WFCAMDEBUG=1
fi

#Check command arguments
if [ $# != 4 ]; then
    echo "Usage : "$0" (Header Name) (Detector ID) (Start No.) (No. of Frames)"
    exit 1
fi

HEAD=$1
DETECTOR=$2
IS=$3
N=$4

# Set Temporary directory
TMPPATHBASE=/tmp/$USER
if [ ! -d $TMPPATHBASE ];then
  mkdir $TMPPATHBASE
fi

COMMANDHEAD=`date +%y%m%d%H%M%S`.mask.$HEAD

TMPPATH=$TMPPATHBASE/$COMMANDHEAD
if [ ! -d $TMPPATH ];then
 mkdir $TMPPATH
fi

# setup filenames
CLCOM=$TMPPATH/cl.com
CLLOG=log/${COMMANDHEAD}.log
CLLOGCOM=`echo $CLLOG | sed 's/.log/.cl/g'`
TMPLOG=$TMPPATH/tmp.log

echo "$0 $1 $2 $3 $4"
echo "# $0 $1 $2 $3 $4" > $CLLOG

echo "# Log will be saved as "$CLLOG
echo "# IRAF commands will be saved as "$CLLOGCOM

# write environment parameters set to logfile
echo WFCAMDATA=$WFCAMDATA >> $CLLOG
echo WFCAMDATE=$WFCAMDATE >> $CLLOG
echo WFCAMLOG=$WFCAMLOG >> $CLLOG
echo WFCAMDEBUG=$WFCAMDEBUG >> $CLLOG
echo MASKRADIUS=$MASKRADIUS >> $CLLOG
echo MASKRADIUS2=$MASKRADIUS2 >> $CLLOG
echo MASKTHRESHO=$MASKTHRESHO >> $CLLOG
echo MASKTHRESHO2=$MASKTHRESHO2 >> $CLLOG

# Check Architecture
ARCH=`uname`
echo "# Running on "`uname -a`
echo "# Running on "`uname -a` >> $CLLOG

# Check IRAF Version
echo "logout" | cl 1> $TMPLOG
IRAFVER=`grep Revision $TMPLOG | sed 's/-/ /g' |  gawk '{split($0,a,"Revision");
print(a[2])}'| gawk '{split($1,a,".");print(a[1]+a[2]/100+a[3]/1000)}'`
echo "# IRAF Version $IRAFVER "
echo "# IRAF Version $IRAFVER " >> $CLLOG

#Start time
if [ $ARCH = "Linux" ]; then
    SECSTART=`date +%s` 
else 
    SECSTART=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

# set filenames
SKYFLAT=flat.fits
DARK=dark${DETECTOR}.fits
FINAL=${HEAD}_${DETECTOR}.fits
MSKY=mask${HEAD}_sky${DETECTOR}.fits
MSKYFLAT=mask${HEAD}_skyflat${DETECTOR}.fits

TMPMASK=tmp.pl
MASKBASE=maskbase.fits

DATAPATH=$WFCAMDATA/wfcam${DETECTOR}/${WFCAMDATE}

TDATAPATH=./tdata
LDATAPATH=./ldata
MASKPATH=./mask

# set detector ID specific information
case $DETECTOR in 
1) DETECTORC=w ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print($11/0.2,-$10/0.2);i++}if(i>=is+n)exit}' $WFCAMLOG > offset.tbl ;;
2) DETECTORC=x ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print(-$10/0.2,-$11/0.2);i++}if(i>=is+n)exit}' $WFCAMLOG > offset.tbl ;;
3) DETECTORC=y ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print(-$11/0.2,$10/0.2);i++}if(i>=is+n)exit}' $WFCAMLOG > offset.tbl ;;
4) DETECTORC=z ; gawk -v is=$IS -v n=$N 'BEGIN{flag=0}{if($1==is){flag=1;i=is}if(flag>0 && $1==i){sub("/","",$10);print($10/0.2,$11/0.2);i++}if(i>=is+n)exit}' $WFCAMLOG > offset.tbl ;;
esac

# create local file table
#gawk -v path=$DATAPATH -v is=$IS -v n=$N -v detector=$DETECTORC 'BEGIN{if(n>0){for(i=is;i<is+n;i++)printf("%s/%s%s_%05d.fit\n",path,detector,date,i)}}' > data.tbl
#gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits/g' > ldata.tbl
#gawk -v path=$LDATAPATH '{n=split($0,a,"/");print(path"/"a[n])}' data.tbl | sed 's/.fit$/.fits[513:1536,513:1536]/g' > ldata-1.tbl

## Check if image viewer is present
echo ## Check if image viewer is present >> $CLLOG

TMPLOG=$TMPPATH/tmp.log
cat << EOF | cl >$TMPLOG
 display ukirt.fits 1 fi+
 logout
EOF

cat $TMPLOG >> $CLLOG

if [ `grep ERROR $TMPLOG | gawk 'END{print(NR)}'` -gt 0 ]; then
 echo "# No image viewer is present."
 echo "# Set WFCAMDISP=2"
 echo "# No image viewer is present." >> $CLLOG
 echo "# Set WFCAMDISP=2" >> $CLLOG
 WFCAMDISP=2
fi

echo "## Detect Objects in $FINAL"
echo "## Detect Objects in $FINAL" >> $CLLOG

cat <<EOF > $CLCOM
 imdel ${TMPPATH}/tmp.fits,${TMPPATH}/mtmp.fits
 magnify $FINAL ${TMPPATH}/tmp.fits 0.5 0.5
 median ${TMPPATH}/tmp.fits ${TMPPATH}/mtmp.fits 3 3
 del ${TMPPATH}/tmp.dat
 imstati ${TMPPATH}/tmp.fits[513:1536,513:1536] format- > ${TMPPATH}/tmp.dat
 logout
EOF

runiraf $CLCOM $CLLOG $CLLOGCOM 

IMSTATUPPER=`gawk '{print($3+$4*3)}' ${TMPPATH}/tmp.dat`
IMSTATLOWER=`gawk '{print($3-$4*3)}' ${TMPPATH}/tmp.dat`

# 2nd imstat
cat <<EOF > $CLCOM
 del ${TMPPATH}/tmp.dat
 imstati ${TMPPATH}/tmp.fits[513:1536,513:1536] format- upper=$IMSTATUPPER lower=$IMSTATLOWER > ${TMPPATH}/tmp.dat
 logout
EOF

runiraf $CLCOM $CLLOG $CLLOGCOM 

IMSTATUPPER=`gawk '{print($3+$4*3)}' ${TMPPATH}/tmp.dat`
IMSTATLOWER=`gawk '{print($3-$4*3)}' ${TMPPATH}/tmp.dat`

cat << EOF > $CLCOM
del tmpstd.dat
imstati ${TMPPATH}/tmp.fits[513:1536,513:1536] format- fields=stddev upper=$IMSTATUPPER lower=$IMSTATLOWER > tmpstd.dat
del tmpsky.dat
imstati ${TMPPATH}/tmp.fits[513:1536,513:1536] format- fields=mean upper=$IMSTATUPPER lower=$IMSTATLOWER > tmpsky.dat
logout
EOF

runiraf $CLCOM $CLLOG $CLLOGCOM 

# calculate offset table for masking
SIGMA=`cat tmpstd.dat`
SKYLV=`cat tmpsky.dat`
DITHX=`gawk 'BEGIN{xmax=-10000;ymax=-10000;xmin=10000;ymin=10000}{if(xmax<$1) xmax=$1;if(ymax<$2) ymax=$2;if(xmin>$1) xmin=$1;if(ymin>$2) ymin=$2;}END{print(int((xmax-xmin)/2)+5)}' offset.tbl`
DITHY=`gawk 'BEGIN{xmax=-10000;ymax=-10000;xmin=10000;ymin=10000}{if(xmax<$1) xmax=$1;if(ymax<$2) ymax=$2;if(xmin>$1) xmin=$1;if(ymin>$2) ymin=$2;}END{print(int((ymax-ymin)/2)+5)}' offset.tbl`
DITHX2=`expr $DITHX + $DITHX`
DITHY2=`expr $DITHY + $DITHY`

# Detect objects
cat << EOF > $CLCOM
 del ${TMPPATH}/tmp.coo
 del ${TMPPATH}/tmpedge.coo
 noao
 digiphot
 apphot
 daofind ${TMPPATH}/mtmp.fits output=${TMPPATH}/tmpedge.coo fwhmpsf=10 thresho=`echo $MASKTHRESHO|gawk '{print($1*2)}'` verify- sigma=$SIGMA sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5
 imshift ${TMPPATH}/mtmp.fits ${TMPPATH}/mtmp.fits $DITHX $DITHY boundary=constant constan=$SKYLV
 imshift ${TMPPATH}/mtmp.fits ${TMPPATH}/mtmp.fits -$DITHX2 0 boundary=constant constan=$SKYLV
 imshift ${TMPPATH}/mtmp.fits ${TMPPATH}/mtmp.fits 0 -$DITHY2 boundary=constant constan=$SKYLV
 imshift ${TMPPATH}/mtmp.fits ${TMPPATH}/mtmp.fits $DITHX $DITHY boundary=constant constan=$SKYLV
 daofind ${TMPPATH}/mtmp.fits output=${TMPPATH}/tmp.coo fwhmpsf=10 thresho=$MASKTHRESHO verify- sigma=$SIGMA sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

if [ $WFCAMDISP -lt 2 ]; then
 cat <<EOF > $CLCOM
  disp ${TMPPATH}/tmp.fits 1 fi+
  tvmark 1 ${TMPPATH}/tmp.coo number+
  tvmark 1 ${TMPPATH}/tmpedge.coo color=200 number+ 
  logout
EOF
 runiraf $CLCOM $CLLOG $CLLOGCOM
fi


# Todo : Need Photometry of each objects?

echo "## Create mask frames"
echo "## Create mask frames" >> $CLLOG

sed '/#/d' ${TMPPATH}/tmp.coo | gawk '{print($1,$2)}'> ${TMPPATH}/tmp2.coo
sed '/#/d' ${TMPPATH}/tmpedge.coo | gawk '{print($1,$2)}'>> ${TMPPATH}/tmp2.coo
gawk 'BEGIN{xmin=10000;ymin=10000}{if(xmin>$1) xmin=$1;if(ymin>$2) ymin=$2;x[NR]=$1;y[NR]=$2}END{for(i=1;i<=NR;i++)print((x[i]-xmin)/2,(y[i]-ymin)/2)}' offset.tbl > zoffset.tbl

cat << EOF > $CLCOM
 imdel $MASKBASE
 imarith $SKYFLAT * 0 $MASKBASE
 logout
EOF

runiraf $CLCOM $CLLOG $CLLOGCOM 

if [ -f mask${DETECTOR}.tbl ]; then
 \rm mask${DETECTOR}.tbl
fi

for dnumber in `gawk '{print(NR)}' offset.tbl`;do
 dnumber2=`echo $dnumber | gawk '{printf("%03d",$1)}'`
# echo $dnumber2
 MASKFILE=$MASKPATH/${DETECTORC}$HEAD_${dnumber2}.pl
 TEXTMASKFILE=$MASKPATH/${DETECTORC}$HEAD_${dnumber2}.coo
 echo $MASKFILE >> mask${DETECTOR}.tbl
 DX=`sed -n ${dnumber}p zoffset.tbl | gawk '{print($1)}'`
 DY=`sed -n ${dnumber}p zoffset.tbl | gawk '{print($2)}'`
# echo $dnumber2 $DX $DY
 gawk -v dx=$DX -v dy=$DY '{x=$1-dx;y=$2-dy;if(x>0 && y>0 && x<2049 && y<2049)print(x,y)}' ${TMPPATH}/tmp2.coo > ${TMPPATH}/tmp3.coo
 cat << EOF > $CLCOM
  imdel $MASKFILE
  imedit $MASKBASE $MASKFILE cursor=${TMPPATH}/tmp3.coo display- radius=$MASKRADIUS value=1 sigma=0 default="e "
  logout
EOF
 runiraf $CLCOM $CLLOG $CLLOGCOM 
 \cp ${TMPPATH}/tmp3.coo $TEXTMASKFILE
done


# Detect objects 2
cat << EOF > $CLCOM
 del ${TMPPATH}/tmp.coo
 noao
 digiphot
 apphot
 daofind ${TMPPATH}/mtmp.fits output=${TMPPATH}/tmp.coo fwhmpsf=10 thresho=$MASKTHRESHO2 verify- sigma=$SIGMA sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5
 logout
EOF
runiraf $CLCOM $CLLOG $CLLOGCOM 

if [ $WFCAMDISP -lt 2 ]; then
 cat <<EOF > $CLCOM
  disp ${TMPPATH}/tmp.fits 1 fi+
  tvmark 1 ${TMPPATH}/tmp.coo number+
  logout
EOF
 runiraf $CLCOM $CLLOG $CLLOGCOM
fi

# Todo : Need Photometry of each objects?

echo "## Create mask frames for bright objects"
echo "## Create mask frames for bright objects" >> $CLLOG

sed '/#/d' ${TMPPATH}/tmp.coo | gawk '{print($1,$2)}'> ${TMPPATH}/tmp2.coo

for dnumber in `gawk '{print(NR)}' offset.tbl`;do
 dnumber2=`echo $dnumber | gawk '{printf("%03d",$1)}'`

 MASKFILE=$MASKPATH/${DETECTORC}$HEAD_${dnumber2}.pl
 TEXTMASKFILE=$MASKPATH/${DETECTORC}$HEAD_${dnumber2}.coo2

 DX=`sed -n ${dnumber}p zoffset.tbl | gawk '{print($1)}'`
 DY=`sed -n ${dnumber}p zoffset.tbl | gawk '{print($2)}'`

 gawk -v dx=$DX -v dy=$DY '{x=$1-dx;y=$2-dy;if(x>0 && y>0 && x<2049 && y<2049)print(x,y)}' ${TMPPATH}/tmp2.coo > ${TMPPATH}/tmp3.coo
 cat << EOF > $CLCOM
  imdel $TMPPATH/$TMPMASK
  imedit $MASKFILE  $TMPPATH/$TMPMASK cursor=${TMPPATH}/tmp3.coo display- radius=$MASKRADIUS2 value=1 sigma=0 default="e "
  imdel $MASKFILE
  imcopy $TMPPATH/$TMPMASK $MASKFILE
  logout
EOF
 \cp ${TMPPATH}/tmp3.coo $TEXTMASKFILE
 runiraf $CLCOM $CLLOG $CLLOGCOM 
done

#END time
if [ $ARCH = "Linux" ] ; then
    SECEND=`date +%s` 
else
    SECEND=`date +"%d %H %M %S"| gawk '{print($1*24*3600+$2*3600+$3*60+$4)}'` 
fi

echo "# End at $SECEND" >> $CLLOG
echo $SECSTART $SECEND | gawk '{printf("# Took %d sec to complete\n",$2-$1)}'
echo $SECSTART $SECEND | gawk '{printf("# Took %d sec to complete\n",$2-$1)}' >> $CLLOG

echo "# Log saved as $CLLOG"
echo "# IRAF commands saved as $CLLOGCOM"
if [ $WFCAMDEBUG -eq 1 ]; then 
  \rm -r $TMPPATH
else 
  echo "# See $TMPPATH for temporary files"
fi

exit 0









